@@ -27,6 +27,7 @@ from dataModel import *
 from tags import *
 from meta import MCache, metaDict2Hash, prettyId, makeId, metaVrr
 from userDb import UserDb
+from platform import guess_prefixes
 
 from whooshSearchEngine import SearchEngine
 from asyncIndex import AsyncIndex
@@ -38,31 +39,36 @@ th_ext_glob=u'*.ki'
 othman=othmanCore()
 
 class ThawabMan (object):
-  def __init__(self,user_prefix,system_prefix="", isMonolithic=True, indexerQueueSize=0):
+  def __init__(self, prefixes=None, isMonolithic=True, indexerQueueSize=0):
     """Create a new Thawab instance given a user writable directory and an optional system-wide read-only directory
 
-  user_prefix can be:
-    user_prefix=os.path.expanduser('~/.thawab')
-    user_prefix=os.path.join([os.path.dirname(sys.argv[0]),'..','data'])
+  prefixes a list of directories all are read-only except the first
+  the first writable directory can be 
+    os.path.expanduser('~/.thawab')
+    os.path.join([os.path.dirname(sys.argv[0]),'..','data'])
   
-  and system_prefix is a system-wide read-only directory like "/usr/share/thawab/"
-
   isMonolithic=True if we should use locks and reconnect to sqlite
   
   indexerQueueSize is the size of threaded index queue (0 infinite, -1 disabled)
 
 the first thing you should do is to call loadMCache()
 """
+    if not prefixes: prefixes=guess_prefixes()
     try:
-      if not os.path.isdir(user_prefix): os.makedirs(user_prefix)
+      if not os.path.isdir(prefixes[0]): os.makedirs(prefixes[0])
     except:
       raise OSError
-    self.prefixes=[os.path.abspath(user_prefix)]
+    self.prefixes=filter(lambda i:os.path.isdir(i), [os.path.realpath(os.path.abspath(p)) for p in prefixes])
+    # make sure it's unique
+    p=self.prefixes[0]
+    s=set(self.prefixes[1:])
+    if p in s: s.remove(p)
+    if len(s)<len(self.prefixes)-1: self.prefixes=[p]+sorted(list(s))
+    #print self.prefixes
     self.othman=othman
     self.__meta=None
-    if system_prefix and os.path.isdir(system_prefix):
-      self.prefixes.append(os.path.abspath(system_prefix))
-    self.assertManagedTree()
+    self.read_only = self.assertManagedTree()
+    self.conf = self.prase_conf()
     self.searchEngine=SearchEngine(self)
     self.user_db=UserDb(self, os.path.join(self.prefixes[0],"user.db") )
     if indexerQueueSize>=0:
@@ -78,8 +84,20 @@ the first thing you should do is to call loadMCache()
       lock1=None
     self.kutubCache=ObjectsCache(lock=lock1)
 
-  def __del__(self):
-    del self.searchEngine
+  def prase_conf(self):
+    r={}
+    fn=os.path.join(self.prefixes[0], 'conf', 'main.txt')
+    if not os.path.exists(fn): return {}
+    try:
+      f=open(fn)
+      t=f.readlines()
+      f.close()
+    except: return {}
+    for l in t:
+      a=l.strip().split("=",1)
+      if len(a)!=2: continue
+      r[a[0].strip()]=a[1].strip()
+    return r
 
   def assertManagedTree(self):
      """create the hierarchy inside the user-managed prefix    
@@ -88,9 +106,12 @@ the first thing you should do is to call loadMCache()
      # conf	application configuration
      # cache	contains the metadata cache for all containers"""
      P=self.prefixes[0]
-     for i in ['db','index','conf','cache', 'tmp']:
+     if not os.access(P, os.W_OK): return False
+     for i in ['db','index','conf','cache', 'tmp', 'themes']:
        p=os.path.join(P,i)
        if not os.path.isdir(p): os.makedirs(p)
+     return True
+
   def mktemp(self):
     h,fn=mkstemp(th_ext, 'THAWAB_',os.path.join(self.prefixes[0],'tmp'))
     return Kitab(fn,True)
@@ -135,7 +156,7 @@ the first thing you should do is to call loadMCache()
   def getKitabByUri(self,uri):
     m=self.getMeta().getByUri(uri)
     if m: return Kitab(uri, th=self, meta=m)
-    return Kitab(uri)
+    return Kitab(uri, th=self)
 
   def getKitabList(self):
     """
@@ -281,6 +302,7 @@ class Kitab(object):
     self.is_tmp = is_tmp
     self.th = th
     self.meta = meta
+    if not meta: self.getMCache()
     if meta and meta.get('originalKitab',None):
       self.originalKi = self.th.getCachedKitabByNameV(meta['originalKitab']+u"-"+meta['originalVersion'])
     else: self.originalKi = None
@@ -318,6 +340,12 @@ class Kitab(object):
       self._cn_h[n] = r
     return r
 
+  def getMCache(self):
+    if not self.th: return None # needs a manager
+    if self.meta: return self.meta
+    self.meta = self.th.getMeta().load_from_uri(self.uri)
+    return self.meta
+
   def setMCache(self, meta):
     # TODO: add more checks
     a=meta.get('author',None)
@@ -330,8 +358,6 @@ class Kitab(object):
     if not y and oy!=None: meta['year']=oy
     if not oy and y!=None: meta['originalYear']=y
     if not meta.get('cache_hash',None): meta['cache_hash']=metaDict2Hash(meta)
-    print SQL_MCACHE_SET
-    print meta
     self.meta=meta
     self.cn().execute(SQL_MCACHE_SET,meta)
 
