@@ -31,8 +31,9 @@ from urllib import unquote
 
 import Thawab.core
 
-from Thawab.webApp import webApp
+from Thawab.webApp import webApp, get_theme_dirs
 from Thawab.shamelaUtils import ShamelaSqlite, shamelaImport
+from Thawab.platform import uri_to_filename
 from paste import httpserver
 
 setsid = getattr(os, 'setsid', None)
@@ -363,8 +364,10 @@ class ThImportWindow(gtk.Window):
           return
         c.flush()
         t_fn=os.path.join(self.main.th.prefixes[0], 'db', u"".join((m['kitab'] + u"-" + m['version'] + Thawab.core.th_ext,)))
-        print "moving %s to %s" % (ki.uri, t_fn)
-        shutil.move(ki.uri, t_fn)
+        #print "moving %s to %s" % (ki.uri, t_fn)
+        try: shutil.move(ki.uri, t_fn)
+        except OSError: print "unable to move converted file." # windows can't move an opened file
+        # FIXME: close ki in a clean way so the above code works in windows
       self.progress_cb(_("Done"), 100.0, show_msg=True)
 
     if db_fn and os.path.exists(db_fn):
@@ -420,7 +423,7 @@ class ThImportWindow(gtk.Window):
     self.ls.append([fn,os.path.basename(fn),0,-1,"Not started"])
 
   def add_uri(self, i):
-    if i.startswith('file://'): f=unquote(i[7:]); self.add_fn(f)
+    if i.startswith('file://'): f=uri_to_filename(unquote(i[7:])); self.add_fn(f)
     else: print "Protocol not supported in [%s]" % i
     
   def drop_data_cb(self, widget, dc, x, y, selection_data, info, t):
@@ -611,7 +614,9 @@ class ThIndexerWindow(gtk.Window):
     self.main=main
     self.connect('delete-event', lambda w,*a: w.hide() or True)
     self.set_title(_('Manage search index'))
-    self.set_type_hint(gtk.gdk.WINDOW_TYPE_HINT_DIALOG)
+    self.set_type_hint(gtk.gdk.WINDOW_TYPE_HINT_UTILITY)
+    self.set_transient_for(main)
+    self.set_position(gtk.WIN_POS_CENTER_ON_PARENT)
     vb=gtk.VBox(False,2); self.add(vb)
     hb=gtk.HBox(False,2); vb.pack_start(hb, False, False, 0)
     self.progress=gtk.ProgressBar()
@@ -632,7 +637,7 @@ class ThIndexerWindow(gtk.Window):
     self.update()
 
   def update(self, *a):
-    if not self.get_visible(): return True
+    if not self.get_property('visible'): return True
     j=self.main.th.asyncIndexer.jobs()
     if j>0:
       self.progress.set_text (_("Indexing ... (%d left)") % j)
@@ -645,11 +650,14 @@ class ThFixesWindow(gtk.Window):
   def __init__(self, main):
     gtk.Window.__init__(self)
     self.set_title(_('Misc. Fixes'))
-    self.set_type_hint(gtk.gdk.WINDOW_TYPE_HINT_DIALOG)
+    self.set_type_hint(gtk.gdk.WINDOW_TYPE_HINT_UTILITY)
     self.set_modal(True)
     self.set_transient_for(main)
+    self.set_position(gtk.WIN_POS_CENTER_ON_PARENT)
+
     self.main=main
     self.connect('delete-event', lambda w,*a: w.hide() or True)
+    self.set_deletable(True)
 
     vb=gtk.VBox(False,2); self.add(vb)
     hb=gtk.HBox(False,2); vb.pack_start(hb, False, False, 0)
@@ -774,7 +782,7 @@ class ThMainWindow(gtk.Window):
     img.set_from_stock(gtk.STOCK_HELP, gtk.ICON_SIZE_BUTTON)
     b=gtk.ToolButton(icon_widget=img, label=_("Help"))
     b.set_tooltip_text(_("Show user manual"))
-    b.connect('clicked', lambda a: self._content.new_tab ("http://127.0.0.1:%d/_files/manual/manual.html" % port))
+    b.connect('clicked', lambda a: self._content.new_tab ("http://127.0.0.1:%d/_theme/manual/manual.html" % port))
     tools.insert(b, -1)
 
     self._content.new_tab()
@@ -822,15 +830,16 @@ THAWAB_HIGH_PORT=18080
 
 def launchServer():
   exedir=os.path.dirname(sys.argv[0])
-  datadir=os.path.join(exedir,'thawab-files')
-  if not os.path.exists(datadir):
-    datadir=os.path.join(exedir,'..','share','thawab','thawab-files')
-  system_prefix=os.path.dirname(datadir) # FIXME: use a smarter place for windows
-  th=Thawab.core.ThawabMan(os.path.expanduser('~/.thawab'), system_prefix=system_prefix, isMonolithic=False)
+  th=Thawab.core.ThawabMan(isMonolithic=False)
+  lookup=[
+    os.path.join(exedir,'thawab-themes'),
+    os.path.join(exedir,'..','share','thawab','thawab-themes'),
+  ]
+  lookup.extend(map(lambda i: os.path.join(i, 'themes'), th.prefixes))
   app=webApp(
-    th,True, 
-    os.path.join(datadir,'templates'),
-    staticBaseDir={'/_files/':os.path.join(datadir,'media')})
+    th,'app', 
+    lookup, th.conf.get('theme', 'default'), '/_theme/',
+  )
   launched=False
   port=THAWAB_HIGH_PORT
   while(not launched):
@@ -846,13 +855,15 @@ def main():
   gettext.install('thawab', ld, unicode=0)
   th, port, server=launchServer()
 
+  gobject.threads_init()
+  gtk.gdk.threads_init()
+
   threading.Thread(target=server.serve_forever, args=()).start()
   while(not server.running): time.sleep(0.25)
-  gtk.gdk.threads_init()
-  #gtk.gdk.threads_enter()
+  gtk.gdk.threads_enter()
   w=ThMainWindow(th, port,server)
-  #gtk.gdk.threads_leave()
   gtk.main()
+  gtk.gdk.threads_leave()
 
 if __name__ == "__main__":
   main()
