@@ -66,7 +66,7 @@ schema_fix_text=re.compile('Memo/Hyperlink',re.I)
 schema_fix_int=re.compile('(Boolean|Byte|Byte|Numeric|Replication ID|(\w+ )?Integer)',re.I)
 sqlite_cols_re=re.compile("\((.*)\)",re.M | re.S)
 no_sql_comments=re.compile('^--.*$',re.M)
-shamela_footers_re=re.compile('^(_{4,})$',re.M)
+shamela_footers_re=re.compile(u'^(¬?_{4,})$',re.M)
 digits_re=re.compile(r'\d+')
 no_w_re=re.compile(ur'[^A-Za-zابتثجحخدذرزسشصضطظعغفقكلمنهوي\s]')
 # one to one transformations that does not change chars order
@@ -103,7 +103,7 @@ def footer_shift_cb(mi):
 
 
 class ShamelaSqlite(object):
-  def __init__(self, bok_fn, cn=None, releaseMajor=0, releaseMinor=0, progress=None, progress_args=[], progress_kw={}, progress_dict=None):
+  def __init__(self, src, cn=None, releaseMajor=0, releaseMinor=0, progress=None, progress_args=[], progress_kw={}, progress_dict=None):
     """import the bok file into sqlite"""
     self.releaseMajor = releaseMajor
     self.releaseMinor = releaseMinor
@@ -112,10 +112,15 @@ class ShamelaSqlite(object):
     self.progress_kw = progress_kw
     self.progress_dict = progress_dict
     self.tables=None
-    self.bok_fn=bok_fn
+    self.tablesFn={}
+    self.src_is_dir=False
+    if os.path.isdir(src): self.sh_prefix=src; self.src_is_dir=True
+    elif os.path.isfile(src): self.bok_fn=src
+    else: raise OSError
     self.metaById={}
     self._blnk={}
     self.xref={}
+    self.encoding_fix_needed=None # True/False or None ie. not yet checked
     self.__bkids=None
     self.__commentaries=None
     self.version,self.tb,self.bkids=self.identify()
@@ -149,22 +154,45 @@ class ShamelaSqlite(object):
     bkid.sort()
     return (3,tb,bkid)
 
-  def getTables(self):
-    if self.tables: return self.tables
-    try: p=Popen(['mdb-tables', '-1',self.bok_fn], 0, stdout=PIPE)
+  def _getTablesInFile(self, fn):
+    try: p=Popen(['mdb-tables', '-1', fn], 0, stdout=PIPE, env={'MDB_JET3_CHARSET':'cp1256', 'MDB_ICONV':'UTF-8'})
     except OSError: raise
-    try: self.tables=p.communicate()[0].strip().split('\n')
+    try: tables=p.communicate()[0].replace('\r','').strip().split('\n')
     except OSError: raise
     r=p.returncode; del p
     if r!=0: raise TypeError
-    self.tables=filter(lambda t: not t.isdigit(), self.tables)
+    tables=filter(lambda t: not t.isdigit(), tables)
+    return tables
+  
+  def _getTablesInBok(self):
+    if self.tables: return self.tables
+    self.tables=self._getTablesInFile(self.bok_fn)
+    self.tablesFn=dict(((t,self.bok_fn) for t in self.tables))
     return self.tables
+  
+  def _getTablesInDir(self):
+    if self.tables: return self.tables
+    self.tables=[]
+    for f in ("main.mdb", "special.mdb"):
+      fn=os.path.join(self.sh_prefix, "Files",f)
+      tb=self._getTablesInFile(fn)
+      self.tables.extend(tb)
+      self.tablesFn.update(dict(((t,fn) for t in tb)))
+    return self.tables
+
+  def _getTableFile(self, tb):
+    if self.src_is_dir: return self.tablesFn[tb]
+    return self.bok_fn
+
+  def getTables(self):
+    if self.tables: return self.tables
+    if self.src_is_dir: return self._getTablesInDir()
+    return self._getTablesInBok()
 
   def __shamela3_fix_insert(self, sql_cmd, prefix="OR IGNORE INTO tmp_"):
     """Internal function used by importTable"""
     if prefix and sql_cmd[0].startswith('INSERT INTO '): sql_cmd[0]='INSERT INTO '+prefix+sql_cmd[0][12:]
     sql=''.join(sql_cmd)
-    #print sql
     self.c.execute(sql)
 
   def __schemaGetCols(self, r):
@@ -177,8 +205,9 @@ class ShamelaSqlite(object):
     """create schema for table"""
     if is_tmp: temp='temp'
     else: temp=''
-    pipe=Popen(['mdb-schema', '-S','-T', Tb, self.bok_fn], 0, stdout=PIPE,env={'MDB_JET3_CHARSET':'cp1256'})
-    r=pipe.communicate()[0]
+    fn=self._getTableFile(Tb)
+    pipe=Popen(['mdb-schema', '-S','-T', Tb, fn], 0, stdout=PIPE,env={'MDB_JET3_CHARSET':'cp1256', 'MDB_ICONV':'UTF-8'})
+    r=pipe.communicate()[0].replace('\r','')
     if pipe.returncode!=0: raise TypeError
     sql=schema_fix_text.sub('TEXT',schema_fix_int.sub('INETEGER',schema_fix_del.sub('',r))).lower()
     sql=sql.replace('create table ',' '.join(('create ',temp,' table ',prefix,)))
@@ -209,17 +238,25 @@ class ShamelaSqlite(object):
     tb_prefix=is_tmp and 'tmp_' or ''
     if Tb in self.imported_tables: return
     self.importTableSchema(Tb, tb, is_tmp, tb_prefix)
-    pipe=Popen(['mdb-export', '-R',';\n'+mark,'-I', self.bok_fn, Tb], 0, stdout=PIPE,env={'MDB_JET3_CHARSET':'cp1256'})
+    fn=self._getTableFile(Tb)
+    pipe=Popen(['mdb-export', '-R',';\n'+mark,'-I', fn, Tb], 0, stdout=PIPE,env={'MDB_JET3_CHARSET':'cp1256', 'MDB_ICONV':'UTF-8'})
     sql_cmd=[]
     prefix=""
     if is_ignore: prefix="OR IGNORE INTO "
     elif is_replace: prefix="OR REPLACE INTO "
     prefix+=tb_prefix
     for l in pipe.stdout:
+      l=l.replace('\r','\n')
+      # output encoding in mdbtools in windows is cp1256, this is a bug in it
+      if self.encoding_fix_needed==None:
+        try: l.decode('UTF-8')
+        except: self.encoding_fix_needed=True; l=l.decode('cp1256')
+        else: self.encoding_fix_needed=False
+      elif self.encoding_fix_needed: l=l.decode('cp1256')
       if l==mark: self.__shamela3_fix_insert(sql_cmd,prefix); sql_cmd=[]
       else: sql_cmd.append(l)
     if len(sql_cmd): self.__shamela3_fix_insert(sql_cmd,prefix); sql_cmd=[]
-    print "waiting child process...",pipe.wait() # TODO: why is this needed
+    pipe.wait() # TODO: why is this needed
     if pipe.returncode!=0: raise TypeError
     del pipe
     self.imported_tables.append(Tb)
@@ -332,7 +369,7 @@ class ShamelaSqlite(object):
         m['classification']=self.classificationByBookId(bkid)
         m['keywords']=u''
         matn_bkid=self._get_matn(bkid)
-        print "%d is sharh for %d" % (bkid, matn_bkid)
+        #print "%d is sharh for %d" % (bkid, matn_bkid)
         if matn_bkid>0:
           matn_m=self.getBookMeta(matn_bkid)
           if matn_m:
@@ -390,6 +427,17 @@ def set_get_xref(xref, h_tags, sh, bkid, pg_id, matn, matnid):
     xref=sh.get_xref(matn, matnid)
     if xref: h_tags['embed.original.section']=xref
 
+ss_re=re.compile(" +")
+re_ss_re=re.compile("( \*){2,}")
+
+def ss(txt):
+  """squeeze spaces"""
+  return ss_re.sub(" ", txt)
+
+def re_ss(txt):
+  """squeeze spaces in re"""
+  return re_ss_re.sub(" *", ss(txt))
+
 
 def shamelaImport(cursor, sh, bkid, footnote_re=ur'\((\d+)\)', body_footnote_re=ur'\((\d+)\)', ft_prefix_len=1, ft_suffix_len=1):
   """
@@ -407,7 +455,7 @@ def shamelaImport(cursor, sh, bkid, footnote_re=ur'\((\d+)\)', body_footnote_re=
   c=sh.c
   # step 0: prepare shorts
   shorts_t=c.execute("SELECT islamshort FROM main WHERE bkid=?", (bkid,)).fetchone()
-  if shorts_t: shorts_t=shorts_t[0]
+  if shorts_t: shorts_t=shorts_t[0] or 0
   else: shorts_t=0
   if shorts_t>1:
     shorts_dict=dict(c.execute("SELECT ramz,nass FROM shorts WHERE bk=?", (bkid,)).fetchall())
@@ -420,6 +468,7 @@ def shamelaImport(cursor, sh, bkid, footnote_re=ur'\((\d+)\)', body_footnote_re=
   # NOTE: we only need page_id,title and depth, sub is only used to sort them
   toc_ls=filter(lambda i: i[2] and i[1], [list(i) for i in r])
   if not toc_ls: raise TypeError # no text in the book
+  if toc_ls[0][0]!=1: toc_ls.insert(0, [1, sh.getBookMeta(bkid)['kitab'].replace('_',' '),toc_ls[0][2]])
   toc_hash=map(lambda i: (i[1][0],i[0]),enumerate(toc_ls))
   # toc_hash.sort(lambda a,b: cmp(a[0],b[0])) # FIXME: this is not needed!
   toc_hash=dict(map(lambda j: (j[0],map(lambda k:k[1], j[1])), groupby(toc_hash,lambda i: i[0])))
@@ -485,30 +534,30 @@ def shamelaImport(cursor, sh, bkid, footnote_re=ur'\((\d+)\)', body_footnote_re=
       h_p=no_w_re.sub(' ', h.translate(sh_normalize_tb)).strip()
       if h_p: # if normalized h_p is not empty
         # NOTE: no need for map h_p on re.escape() because it does not contain special chars
-        h_re_entire_line=re.compile(ur"^\s*%s\s*$" % ur" *".join(list(h_p)), re.M)
+        h_re_entire_line=re.compile(re_ss(ur"^\s*%s\s*$" % ur" *".join(list(h_p))), re.M)
         if _shamelaFindHeadings(txt, page_id, d, h, h_re_entire_line, ix, j, 2): continue
 
       if not txt_no_d: txt_no_d=txt.translate(sh_digits_to_spaces_tb)
       h_p_no_d=h_p.translate(sh_digits_to_spaces_tb).strip()
       if h_p_no_d:
-        h_re_entire_line_no_d=re.compile(ur"^\s*%s\s*$" % ur" *".join(list(h_p_no_d)), re.M)
+        h_re_entire_line_no_d=re.compile(re_ss(ur"^\s*%s\s*$" % ur" *".join(list(h_p_no_d))), re.M)
         if _shamelaFindHeadings(txt_no_d, page_id, d, h, h_re_entire_line_no_d, ix, j, 3): continue
 
       # at the beginning of the line
       if _shamelaFindExactHeadings(page_txt, page_id, "\n%s", d, h, ix,j, 4): continue
       if h_p:
-        h_re_line_start=re.compile(ur"^\s*%s\s*" % ur" *".join(list(h_p)), re.M)
+        h_re_line_start=re.compile(re_ss(ur"^\s*%s\s*" % ur" *".join(list(h_p))), re.M)
         if _shamelaFindHeadings(txt, page_id, d, h, h_re_line_start, ix, j, 5): continue
       if h_p_no_d:
-        h_re_line_start_no_d=re.compile(ur"^\s*%s\s*" % ur" *".join(list(h_p_no_d)), re.M)
+        h_re_line_start_no_d=re.compile(re_ss(ur"^\s*%s\s*" % ur" *".join(list(h_p_no_d))), re.M)
         if _shamelaFindHeadings(txt_no_d, page_id, d, h, h_re_line_start_no_d, ix, j, 6): continue
       # any where in the line
       if _shamelaFindExactHeadings(page_txt, page_id, "%s", d, h, ix,j, 7): continue
       if h_p:
-        h_re_any_ware=re.compile(ur"\s*%s\s*" % ur" *".join(list(h_p)), re.M)
+        h_re_any_ware=re.compile(re_ss(ur"\s*%s\s*" % ur" *".join(list(h_p))), re.M)
         if _shamelaFindHeadings(txt, page_id, d, h, h_re_any_ware, ix, j, 8): continue
       if h_p_no_d:
-        h_re_any_ware_no_d=re.compile(ur"\s*%s\s*" % ur" *".join(list(h_p_no_d)), re.M)
+        h_re_any_ware_no_d=re.compile(re_ss(ur"\s*%s\s*" % ur" *".join(list(h_p_no_d))), re.M)
         if _shamelaFindHeadings(txt_no_d, page_id, d, h, h_re_any_ware, ix, j, 9): continue
       # if we reached here then head is not found
       # place it just after last one
@@ -607,7 +656,6 @@ def shamelaImport(cursor, sh, bkid, footnote_re=ur'\((\d+)\)', body_footnote_re=
     #  for j,k in footnotes_cnd:
     #    print "j=[%s] k=[%s]" % (j,k)
     #  # raise KeyError
-
     if toc_hash.has_key(pg_id):
       hno_pop_needed=False
     elif hno!=None and hno!=last_hno:
@@ -694,7 +742,6 @@ def shamelaImport(cursor, sh, bkid, footnote_re=ur'\((\d+)\)', body_footnote_re=
     depths.append(f.depth)
     #last=pg_body[f.end:]+'\n'
     last=shamela_shift_footers_re.sub(footer_shift_cb, pg_body[f.end:]+'\n')
-    if footnotes_cnd and pg_id!=154: print " *** stall footnotes at pg_id=",pg_id; # raise
     if footnotes_cnd:
       last+="\n==========[\n"+pop_footers(footnotes_cnd)+"\n]==========\n"
     
